--- 模块功能：串口1功能测试
-- @author openLuat
-- @module uart.uartTask
-- @license MIT
-- @copyright openLuat
-- @release 2018.03.27

module(...,package.seeall)

require "utils"
require "pm"
require "sys"
require "log"
require "nvm"
--[[
功能定义：
uart按照帧结构接收外围设备的输入，收到正确的指令后，回复ASCII字符串

帧结构如下：
帧头：1字节，0x01表示扫描指令，0x02表示控制GPIO命令，0x03表示控制端口命令
帧体：字节不固定，跟帧头有关
帧尾：1字节，固定为0xC0

收到的指令帧头为0x01时，回复"CMD_SCANNER\r\n"给外围设备；例如接收到0x01 0xC0两个字节，就回复"CMD_SCANNER\r\n"
收到的指令帧头为0x02时，回复"CMD_GPIO\r\n"给外围设备；例如接收到0x02 0xC0两个字节，就回复"CMD_GPIO\r\n"
收到的指令帧头为0x03时，回复"CMD_PORT\r\n"给外围设备；例如接收到0x03 0xC0两个字节，就回复"CMD_PORT\r\n"
收到的指令帧头为其余数据时，回复"CMD_ERROR\r\n"给外围设备；例如接收到0x04 0xC0两个字节，就回复"CMD_ERROR\r\n"
]]


--串口ID,1对应uart1
--如果要修改为uart2，把UART_ID赋值为2即可
-- air720UH 可用串口1和2，rfid串口用1
-- air724UH 可用串口1、2、3，rfid串口用3
local UART_ID = nvm.get("rfidUartId")
local UART_ID_BAUD_RATE = nvm.get("rfidUartBaudRate")
local rfidOfflineTimeout = nvm.get("rfidOfflineTimeout")
local rfidMinDiffTime = nvm.get("rfidMinDiffTime") or 3

--local UART_ID = 3
--帧头以及帧尾
local FRM_HEAD,FRM_TAIL = 0xaa55,0x0d0a
--rfid 去重缓存的超时时间
local  rfid_buffer_timeout = 20
local  rfid_buffer_max_count = 100
--串口读到的数据缓冲区
local rdbuf = ""
--rfid串口初始化定参数时器
local rfid_uart_config_init_timer_id = nil
local rfidType = nvm.get("rfidType")
local timerSyncResult = false

--是否厂测读rfid标签模式
local isFactoryTestReadRfidMode = false
--厂测模式读rifd号
local factoryTestReadRfid = ""
--是否已开始心跳
local isStartHeartbeat = false

local isCmdReadRfidMode = false
local cmdReadRfid = ""

require "misc"

local function getTime()
    local tm = misc.getClock()
    return string.format("%04d/%02d/%02d,%02d:%02d:%02d", tm.year, tm.month, tm.day, tm.hour, tm.min, tm.sec)
end

--初始化rfid缓冲池
local start_time = os.time()
local rfids_buf = {["start_time"]=start_time,["rfid_count"]=0,["buf"]={}}
--rfid缓冲池最大存储标签个数
local rfidsBufMaxCount = 3000


--任联rfid数据解析
local function parse_rfid_renlian(body)
    if not body then return false end
    if body:len() ~= 9 then 
        log.info("parse_rfid_renlian fail,body.length != 9")
        return false 
    end
    local id = string.sub(body,1,6):toHex()
    local type = tonumber(string.byte(body,7))
    local status = tonumber(string.byte(body,8))
    local rssi = tonumber(string.byte(body,9))
    local rfid = {["id"]=id,["type"]=type,["status"]=status,["rssi"]=rssi}
    return rfid
end

--任联rfid数据解析
local function parse_rfid_tiandiren(body)
    if not body then return false end
    if body:len() ~= 18 then 
        log.info("parse_rfid_tiandiren fail,body.length != 18",body:toHex())
        return false 
    end
    local id = string.sub(body,5,8):toHex()
    if not id then 
        log.info("parse_rfid_tiandiren fail,id empty! body:",body:toHex())
        return false
    end
    local sn = string.byte(body,2)
    local dev_type = string.byte(body,3)*16 + string.byte(body,4)
    local cmd_type = string.byte(body,9)
    local crc = string.byte(body,15) + string.byte(body,16) * 16
    local content = string.sub(body,10,14):toHex()
    local status = string.byte(body,17)
    local rssi = string.byte(body,18)
    --local rfid = {["id"]=id,["cmd_type"]=cmd_type,["dev_type"]=dev_type,["sn"]=sn}
    local rfid = {["id"]=id,["type"]=cmd_type,["status"]=status,["rssi"]=rssi,["dev_type"]=dev_type,["sn"]=sn,["content"]=content}
    return rfid
end

--[[
函数名：parse_renlian
功能  ：按照帧结构解析处理一条完整的帧数据
参数  ：
        data：所有未处理的数据
返回值：第一个返回值是一条完整帧报文的处理结果，第二个返回值是未处理的数据
]]
local function parse_renlian(data)
    if not data then return end    
    if data:len() < 14 then 
        return  false, data
    end
    
    local frm_head = tonumber(string.format( "%02x",string.byte(data,1)) .. string.format( "%02x",string.byte(data,2)),16)
    
    local tail = string.find(data,string.char(0x0d)..string.char(0x0a))
    if not tail then 
        log.info("end char:0x0d0a not found,wait for more ... ", data:toHex())
        return false,data 
    end 
    if frm_head ~= FRM_HEAD then
        log.info("FRM_HEAD ERROR,drop it, frm_head = ", frm_head, ", FRM_HEAD = ", FRM_HEAD)
        return false,string.sub(data,tail+2,-1) 
    end

    local frm_len = tonumber(string.byte(data,3))

    local body,result = string.sub(data,4,tail-1)
    --检测body完整性
    if (frm_len-2) ~= body:len() then
        log.info("frm body length error, frm_len-2 = ",frm_len-2,", body:len() = ", body:len(), ", body:toHex() = ", body:toHex())
        return false,string.sub(data,tail+2,-1) 
    end
    
    --log.info("uart1Task.parse",data:toHex(),frm_head,body:toHex())
    
    --解析rfid
    local rfid_info = parse_rfid_renlian(body)
    if not rfid_info then 
        log.info("parse_rfid_renlian() failed!")
        return false,string.sub(data,tail+2,-1)
    end

    --检测到超时时，发布数据并重置 rfids_buf
    local cur_time = os.time()
    if (cur_time - rfids_buf.start_time >= rfid_buffer_timeout) or (rfids_buf.rfid_count >= rfid_buffer_max_count) then
        if rfids_buf.rfid_count > 0 then
            --log.info(">>>>>>> uart1Task.parse pub_packet_data", cur_time)
            sys.publish("pub_packet_data_renlian", rfids_buf)
        end
        rfids_buf = {["start_time"]=cur_time,["rfid_count"]=0,["buf"]={}}
    end

    --新增数据
    if not rfids_buf["buf"][rfid_info.id] then
        --log.info(">>>>>>> push rfid to buf,rfid:", rfid_info.id)
        rfids_buf.rfid_count = rfids_buf.rfid_count + 1
        rfids_buf["buf"][rfid_info.id] = {["time"]=cur_time,["recv_count"]=1,["rssi"]=rfid_info.rssi,["type"]=rfid_info.type,["status"]=rfid_info.status}
        return true,string.sub(data,tail+2,-1)
    end

    --更新数据
    local recv_count = rfids_buf["buf"][rfid_info.id]["recv_count"] + 1
    rfids_buf["buf"][rfid_info.id] = {["time"]=cur_time,["recv_count"]=recv_count,["rssi"]=rfid_info.rssi,["type"]=rfid_info.type,["status"]=rfid_info.status}
    --log.info(">>>>>>> update rfid_buf,rfid:", rfid_info.id, ", recv_count:", recv_count)

    return true,string.sub(data,tail+2,-1)    
end


--[[
函数名：parse_tiandiren
功能  ：按照帧结构解析处理一条完整的帧数据
参数  ：
        data：所有未处理的数据
返回值：第一个返回值是一条完整帧报文的处理结果，第二个返回值是未处理的数据
]]
local function parse_tiandiren(data)
    local rfidRssiFilterVal = nvm.get("rfidRssiFilterVal")
    local enableFilterTdrRfid = nvm.get("enableFilterTdrRfid")
    local onlyDevTypeVal = nvm.get("onlyDevTypeVal")
    if not data then return end    
    if data:len() < 22 then 
        --log.info("uartTaskV2.parse_tiandiren","data length < 22", data:toHex())
        return  false, data
    end
    
    local frm_head = tonumber(string.format( "%02x",string.byte(data,1)) .. string.format( "%02x",string.byte(data,2)),16)
    
    local tail = string.find(data,string.char(0x0d)..string.char(0x0a))
    if not tail then 
        log.info("uartTaskV2.parse_tiandiren","end char:0x0d0a not found,wait for more ... ", data:toHex())
        return false,data 
    end 
    if frm_head ~= FRM_HEAD then
        log.info("uartTaskV2.parse_tiandiren","FRM_HEAD ERROR,drop it, frm_head = ", frm_head, ", FRM_HEAD = ", FRM_HEAD)
        return false,string.sub(data,tail+2,-1) 
    end

    local frm_len = tonumber(string.byte(data,3))

    local body,result = string.sub(data,4,tail-1)
    --检测body完整性
    if (frm_len-2) ~= body:len() then
        log.info("uartTaskV2.parse_tiandiren","frm body length error, frm_len-2 = ",frm_len-2,", body:len() = ", body:len(), ", body:toHex() = ", body:toHex())
        return false,string.sub(data,tail+2,-1) 
    end
    
    --log.info("uart1Task.parse",data:toHex(),frm_head,body:toHex())
    
    --解析rfid
    local rfid_info = parse_rfid_tiandiren(body)
    if not rfid_info then 
        log.info("uartTaskV2.parse_tiandiren", "parse_rfid_tiandiren() failed!",data:toHex())
        return false,string.sub(data,tail+2,-1)
    end

    if rfidRssiFilterVal > 0 and rfid_info.rssi > rfidRssiFilterVal then  
        log.info("uartTaskV2.parse_tiandiren", "rfid_info.rssi > ".. rfidRssiFilterVal..", drop it! rfid_info.rssi =", rfid_info.rssi, "rfid:", rfid_info.id)
        return false,string.sub(data,tail+2,-1)
    end
    -- 过滤原天地人的标签，根据运维平台过滤开关是否过滤  不是3开头，十进制长度不足7位，不上报
    if enableFilterTdrRfid == 1 and ( (#tostring(tonumber(rfid_info.id, 16)) ~= 7) or ( tonumber(string.sub(tonumber(rfid_info.id, 16), 1, 1) )  ~= 3 ) ) then 
        local id_decnum = tonumber(rfid_info.id, 16)
        log.info("uartTaskV2.parse_tiandiren", "rfid_info.id head ~= ".. 3 ..", drop it! rfid_info.id =", rfid_info.id, "id_decnum:", id_decnum)
        return false,string.sub(data,tail+2,-1)
    end
    -- 只报配置dev_type的标签
    if onlyDevTypeVal ~= "" then 
        local transDevTypeVal = tonumber(string.sub(onlyDevTypeVal, 0, 2), 16)*16 + tonumber(string.sub(onlyDevTypeVal, 3, -1), 16)
        if rfid_info.dev_type ~= transDevTypeVal then
            log.info("uartTaskV2.parse_tiandiren", "rfid_info.dev_type ~= ".. onlyDevTypeVal ..", drop it! rfid_info.id =", rfid_info.id, "rfid_info.dev_type:", rfid_info.dev_type,"transDevTypeVal:", transDevTypeVal)
            return false,string.sub(data,tail+2,-1)
        end
    end

    

    local cur_time = os.time()
    
    --进入厂测读rfid标签信号模式
    if isFactoryTestReadRfidMode then
        --读到rfid标签信号后，退出厂测模式
        if factoryTestReadRfid == rfid_info.id then
            sys.publish("factory_test_read_rfid_success",{rssi=rfid_info.rssi,rfid=rfid_info.id})
            isFactoryTestMode = false
            factoryTestReadRfid = ""
        end
        return true,string.sub(data,tail+2,-1)
    end
    --进入指令读rfid标签信号模式
    if isCmdReadRfidMode then
        --读到rfid标签信号后，退出读标签模式
        if cmdReadRfid == rfid_info.id then
            log.info("uartTaskV2.parse_tiandiren", "rfid_info:", json.encode(rfid_info))
            sys.publish("cmd_read_rfid_success",{rssi=rfid_info.rssi,rfid=rfid_info.id})
            isCmdReadRfidMode = false
            cmdReadRfid = ""
        end
        return true,string.sub(data,tail+2,-1)
    end
    --新增数据
    if not rfids_buf["buf"][rfid_info.id] then
        --log.info(">>>>>>> push rfid to buf,rfid:", rfid_info.id)
        if rfids_buf.rfid_count > rfidsBufMaxCount then
            log.warn("uartTaskV2.parse_tiandiren", "rfids_buf.rfid_count > ", rfidsBufMaxCount, ", drop it", rfid_info.id)
            return true,string.sub(data,tail+2,-1)
        end
        rfids_buf.rfid_count = rfids_buf.rfid_count + 1
        rfids_buf["buf"][rfid_info.id] = {["id"]=rfid_info.id,["first_time"]=cur_time,["time"]=cur_time,["recv_count"]=1,["rssi"]=rfid_info.rssi,["type"]=rfid_info.type,["status"]=rfid_info.status,["dev_type"]=rfid_info.dev_type,["sn"]=rfid_info.sn,["content"]=content}
        
        --发布rfid首个信号
        sys.publish("pub_packet_data_tiandiren_rfid_info", rfids_buf["buf"][rfid_info.id])

        return true,string.sub(data,tail+2,-1)
    end
    local first_time = rfids_buf["buf"][rfid_info.id]["first_time"]
    --更新数据
    local recv_count = rfids_buf["buf"][rfid_info.id]["recv_count"] + 1
    rfids_buf["buf"][rfid_info.id] = {["id"]=rfid_info.id,["first_time"]=first_time,["time"]=cur_time,["recv_count"]=recv_count,["rssi"]=rfid_info.rssi,["type"]=rfid_info.type,["status"]=rfid_info.status,["dev_type"]=rfid_info.dev_type,["sn"]=rfid_info.sn,["content"]=content}
    --log.info(">>>>>>> update rfid_buf,rfid:", rfid_info.id, ", recv_count:", recv_count)

    return true,string.sub(data,tail+2,-1)    
end


--[[
函数名：proc
功能  ：处理从串口读到的数据
参数  ：
        data：当前一次从串口读到的数据
返回值：无
]]
local function proc(data)
    if not data or string.len(data) == 0 then return end
    if not rfidType then 
        log.info("uartTask.proc","rfidType undefined!!")
        rfidType = nvm.get("rfidType")
        return false
    end
    --追加到缓冲区
    rdbuf = rdbuf..data    
    
    local result,unproc
    unproc = rdbuf
    --根据帧结构循环解析未处理过的数据
    while true do
        if rfidType == 'renlian' then
            result,unproc = parse_renlian(unproc)
        elseif rfidType == 'tiandiren' then
            result,unproc = parse_tiandiren(unproc)
        else
            log.info("uartTask.proc","rfidType invalid!!")
            return false
        end
        if not unproc or unproc == "" or not result then
            break
        end
    end

    rdbuf = unproc or ""
    if #rdbuf > 1000 then
        log.warn("uartTaskV2.proc","rdbuf length > 1000!! rdbuf:", #rdbuf)
    end
end

--[[
函数名：read
功能  ：读取串口接收到的数据
参数  ：无
返回值：无
]]
local function read()
    local data = ""
    --底层core中，串口收到数据时：
    --如果接收缓冲区为空，则会以中断方式通知Lua脚本收到了新数据；
    --如果接收缓冲器不为空，则不会通知Lua脚本
    --所以Lua脚本中收到中断读串口数据时，每次都要把接收缓冲区中的数据全部读出，这样才能保证底层core中的新数据中断上来，此read函数中的while语句中就保证了这一点
    while true do        
        data = uart.read(UART_ID,"*l")
        if not data or string.len(data) == 0 then break end
        --打开下面的打印会耗时
        --log.info("uartTask.read bin",data)
        --log.info("uartTask.read hex",data:toHex())
        
        --处理串口收到的rfid数据
        proc(data)
    end
end

--[[
函数名：write
功能  ：通过串口发送数据
参数  ：
        s：要发送的数据
返回值：无
]]
function write(s)
    log.info("uartTask.write",s)
    uart.write(UART_ID,s.."\r\n")
end

local function writeOk()
    log.info("uartTask.writeOk")
end

--[[
--检测 rfids_buf是否超时
sys.timerLoopStart(function ()
    if not timerSyncResult then
        log.info("uartTask.timerLoopStart","wait TIMER_SYNC_SUCCESS message ...")
        return false
    end
    if not rfidType then 
        log.info("uartTask.timerLoopStart","rfidType undefined!!")
        rfidType = nvm.get("rfidType")
        return false
    end
    --log.info(">>>>>>> uart1Task.timerLoopStart ")
    --检测到超时时，发布数据并重置 rfids_buf
    local cur_time = os.time()
    if (cur_time - rfids_buf.start_time >= rfid_buffer_timeout) or (rfids_buf.rfid_count >= rfid_buffer_max_count) then
        if rfids_buf.rfid_count > 0 then
            if rfidType == 'renlian' then
                --log.info("uartTask.timerLoopStart",">>>>>>> uart1Task.timerLoopStart pub_packet_data_renlian", cur_time)
                sys.publish("pub_packet_data_renlian", rfids_buf)
            elseif rfidType == 'tiandiren' then
                sys.publish("pub_packet_data_tiandiren", rfids_buf)
                --sys.publish("pub_packet_data_renlian", rfids_buf)
            else
                log.info("uartTask.timerLoopStart","rfidType invalid!!")
                return false
            end
        end
        rfids_buf = {["start_time"]=cur_time,["rfid_count"]=0,["buf"]={}}
    end
end, 100)
]]

--等待时间同步成功消息通知
sys.subscribe("TIMER_SYNC_SUCCESS", function ( ... )
    log.info("uartTask.subscribe","TIMER_SYNC_SUCCESS")
    timerSyncResult = true
end)

--启动厂测读rfid标签模式，读取指定rfid标签信号
sys.subscribe("factory_test_start_read_rfid", function ( data )
    if data.rfid then
        log.info("uartTaskV2.subscribe","factory test mode,start read rifd:",data.rfid)
        isFactoryTestReadRfidMode = true
        factoryTestReadRfid = data.rfid
        if not isStartHeartbeat then
            sys.publish("MESSAGE_TYPE_START_HEARTBEAT")
        end
    else
        log.error("uartTaskV2.subscribe.factory_test_start_read_rfid","data.rfid not existed!!")
    end
end)

--退出厂测读rfid标签模式
sys.subscribe("factory_test_stop_read_rfid", function ( )
    log.info("uartTaskV2.subscribe.factory_test_stop_read_rfid","factory test stop read rifd")
    isFactoryTestReadRfidMode = false
    factoryTestReadRfid = ""
end)

--启动读rfid标签模式，读取指定rfid标签信号
sys.subscribe("cmd_start_read_rfid", function ( data )
    if not isStartHeartbeat then
        sys.publish("MESSAGE_TYPE_START_HEARTBEAT")
    end
    if data.rfid then
        log.info("uartTaskV2.subscribe.cmd_start_read_rfid","cmd start read rifd:",data.rfid)
        isCmdReadRfidMode = true
        cmdReadRfid = data.rfid
    else
        log.error("uartTaskV2.subscribe.cmd_start_read_rfid","data.rfid not existed!!")
    end
end)

--退出厂测读rfid标签模式
sys.subscribe("cmd_stop_read_rfid", function ( )
    log.info("uartTaskV2.subscribe.cmd_stop_read_rfid","cmd stop read rifd")
    isCmdReadRfidMode = false
    cmdReadRfid = ""
end)

--保持系统处于唤醒状态，此处只是为了测试需要，所以此模块没有地方调用pm.sleep("uartTask")休眠，不会进入低功耗休眠状态
--在开发“要求功耗低”的项目时，一定要想办法保证pm.wake("uartTask")后，在不需要串口时调用pm.sleep("uartTask")
pm.wake("uartTask")

--sys.timerStart(function ( ... )
    

--初始化rfid串口配置参数
rfid_uart_config_init_timer_id = sys.timerLoopStart(function ()
    if not UART_ID then
        log.info("UART_ID empty, attempt from nvm.get")
        UART_ID = nvm.get("rfidUartId")
        return false
    end
    if not UART_ID_BAUD_RATE then
        log.info("UART_ID_BAUD_RATE empty, attempt from nvm.get")
        UART_ID_BAUD_RATE = nvm.get("rfidUartBaudRate")
        return false
    end
    if not rfid_uart_config_init_timer_id then
        log.info("rfid_uart_config_init_timer_id empty")
        return false
    end
    log.info("init rfid uart","UART_ID:",UART_ID,"UART_ID_BAUD_RATE:",UART_ID_BAUD_RATE)
    --注册串口的数据接收函数，串口收到数据后，会以中断方式，调用read接口读取数据
    --uart.on(UART_ID,"receive",read)
    uart.on(UART_ID,"receive",function() sys.publish("UART_RECEIVE") end)
    --注册串口的数据发送通知函数
    uart.on(UART_ID,"sent",writeOk)

    --配置并且打开串口
    --波特率，可选1200，2400，4800，9600，10400，14400，19200，28800，38400，57600，115200，230400，460800，921600
    --uart.setup(UART_ID,115200,8,uart.PAR_NONE,uart.STOP_1)
    uart.setup(UART_ID,UART_ID_BAUD_RATE,8,uart.PAR_NONE,uart.STOP_1)
    --如果需要打开“串口发送数据完成后，通过异步消息通知”的功能，则使用下面的这行setup，注释掉上面的一行setup
    --uart.setup(UART_ID,115200,8,uart.PAR_NONE,uart.STOP_1,nil,1)
    log.info("uart timer loop stop","rfid_uart_config_init_timer_id:",rfid_uart_config_init_timer_id)
    sys.timerStop(rfid_uart_config_init_timer_id)
end, 1000)

--end, 10000)


local function taskRead()
    --等心跳开启时，再开始接收串口rfid数据
    while true do
        log.info("uartTaskV2.taskRead","wait MESSAGE_TYPE_START_HEARTBEAT message ...")
        local res,data = sys.waitUntil("MESSAGE_TYPE_START_HEARTBEAT", 3600*1000)
        if res then
            log.info("uartTaskV2.taskRead","receive MESSAGE_TYPE_START_HEARTBEAT messages, start UART_RECEIVE task...")
            isStartHeartbeat = true
            break
        else
            log.info("uartTaskV2.taskRead","wait MESSAGE_TYPE_START_HEARTBEAT timeout")
        end
    end
    local cacheData = ""
    local count = 0
    while true do
        cacheData = uart.read(UART_ID,"*l")
        if cacheData == "" then            
            if not sys.waitUntil("UART_RECEIVE",100) then
                --log.info("uartTask.taskRead","100ms no data")
            end
        else
            --log.info("uartTask.read hex",cacheData:toHex())
            proc(cacheData)
            
            cacheData = ""
            count = count + 1
            if count > 1200 then
                sys.wait(50)
                count = 0
            end
        end
    end
end
--启动串口数据接收任务
sys.taskInit(taskRead)


sys.timerLoopStart(function()
    log.info("uartTaskV2.timerLoopStart","打印占用的内存:", _G.collectgarbage("count"))-- 打印占用的RAM
    log.info("uartTaskV2.timerLoopStart","打印可用的空间", rtos.get_fs_free_size())-- 打印剩余FALSH，单位Byte
    log.info("uartTaskV2.timerLoopStart", "rfidMinDiffTime:",rfidMinDiffTime)
end, 10000)

--rfid离线检测任务
sys.taskInit(function()
    while true do
        while true do
            if rfids_buf["rfid_count"] > 0 then
                break
            else
                log.info("uartTaskV2.rfidOfflineCheckTask","rfids_buf empty,wait ...")
                sys.wait(1000)
            end
        end
        
        local now = os.time()
        --log.info("uartTaskV2.rfidOfflineCheckTask","start time:",now)
        local tmp = {["rfid_count"]=0,["buf"]={}}
        local checkCount = 0
        local onlineCount = 0
        local offlineCount = 0
        local totalCount = 0
        for rfid, rfid_info in pairs(rfids_buf["buf"]) do
            totalCount = totalCount + 1
            if rfid_info.time and rfid_info.time > 0 then
                checkCount = checkCount + 1
                if now - rfid_info.time <= rfidOfflineTimeout then
                    if rfid == "953651BE" then
                        --log.info("uartTaskV2.rfidOfflineCheckTask","rfid is online,skip! rfid:", rfid, "rfid_info.time:", rfid_info.time)
                    end
                    tmp["buf"][rfid] = rfid_info
                    tmp["rfid_count"] = tmp["rfid_count"] + 1
                    onlineCount = onlineCount + 1
                else
                    --首末次信号时间差超过最小时间差值，才认为是有效的末次信号
                    if rfid_info.time - rfid_info.first_time > rfidMinDiffTime then
                        offlineCount = offlineCount + 1
                        if rfid == "953651BE" then
                            log.info("uartTaskV2.rfidOfflineCheckTask","rfid offline! rfid:", rfid, "rfid_info.time:", rfid_info.time, "now:", now, "rfidOfflineTimeout:", rfidOfflineTimeout)
                        end
                        --上报离线后，末次信号
                        if rfids_buf["buf"][rfid] then
                            sys.publish("pub_packet_data_tiandiren_rfid_info", rfids_buf["buf"][rfid])
                            if offlineCount >= 100 and offlineCount%100 == 0 then
                                --log.info("uartTaskV2.rfidOfflineCheckTask","wait 3s")
                                sys.wait(50)
                            end
                        end
                    else
                        log.info("uartTaskV2.rfidOfflineCheckTask","drop it, because: rfid_info.time - rfid_info.first_time <=",rfidMinDiffTime)
                    end
                end
            end
        end
        rfids_buf = tmp
        log.info("uartTaskV2.rfidOfflineCheckTask", "totalCount:",totalCount, "checkCount:",checkCount, "onlineCount:",onlineCount, "offlineCount:",offlineCount)
        sys.wait(rfidOfflineTimeout*1000 + 2000)
    end
end)
